/*
    *
    * This file is a part of CoreImage.
    * An image viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 2 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QListWidgetItem>
#include <QStandardPaths>
#include <QImageReader>
#include <QImageWriter>
#include <QFileInfo>
#include <QShortcut>
#include <QTimer>

#include <cprime/themefunc.h>
#include <cprime/cprime.h>
#include <cprime/appopenfunc.h>
#include <cprime/pinit.h>
#include <cprime/trashmanager.h>
#include <cprime/shareit.h>

#include "coreimage.h"
#include "ui_coreimage.h"


coreimage::coreimage(QWidget *parent) : QWidget(parent), ui(new Ui::coreimage)
    , workFilePath("")
    , slideShow(false)
    , slideShowTimer(nullptr)

{
    ui->setupUi(this);

    loadSettings();
	startSetup();
    startMode();
	shortcuts();
	getImageFilters();
}

coreimage::~coreimage()
{
    if (slideShowTimer) {
		delete slideShowTimer;
    }

	delete ui;
}

void coreimage::sendFiles(const QStringList &paths)
{
    if (paths.count()) {
        QString str = paths.first();

        if (CPrime::FileUtils::isFile(str)) {
            loadImage(str);
        } else {
            qDebug() << "func(sendFiles) : Error : Invalid file path sent :" << str;
        }
    }
}

bool coreimage::loadImage(QString filePath)
{
    bool imageLoaded = false;

    if (!filePath.count()) {
        CPrime::InfoFunc::messageEngine("CoreImage", "coreimage", "Empty file name!!!", "Select a valid file", this);
        return imageLoaded;
    }

    if (ui->page->currentIndex() != 1) {
        ui->page->setCurrentIndex(1);
    }

    imageLoaded = ui->viewWidget->loadImage(filePath);
    QString fileName = QFileInfo(filePath).fileName();

    if (imageLoaded) {
        workFilePath = filePath;
        setWindowTitle(fileName + " - CoreImage");
        if (not touch) {
            QFileInfo info(filePath);
            QString n = info.fileName();
            QString h = QString::number(ui->viewWidget->asImage().height()) ;
            QString w = QString::number(ui->viewWidget->asImage().width()) ;
            QString s = CPrime::FileFunc::formatSize(static_cast<quint64>(info.size()));
            QString typ = info.suffix();
            ui->propertisesL->setText(QString("%1; %2 px x %3 px; %4; %5;")
                                      .arg(n).arg(w).arg(h).arg(s).arg(typ));
        }
    } else {
        QString mess = tr("Could not load image <b>%1</b>").arg(fileName) ;
        CPrime::InfoFunc::messageEngine("CoreImage", "coreimage", "Failed to load", mess, this);
    }

    return imageLoaded;
}

void coreimage::closeEvent(QCloseEvent *event)
{
    CPrime::InfoFunc::saveToRecent("coreimage", workFilePath);
    event->accept();
}

/**
 * @brief Loads application settings
 */
void coreimage::loadSettings()
{
    sideViewSize = sm->value("CoreApps", "SideViewSize");
    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");
    touch = sm->value("CoreApps", "TouchMode");
    tabletMode = sm->value("CoreApps", "TabletMode");
}

/**
 * @brief Setup ui elements
 */
void coreimage::startSetup()
{
    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, Qt::transparent);
    setPalette(pltt);

    imagesList.clear();
    openFilters.clear();
    saveFilters.clear();

    // Zoom label
	zoomLbl = new QLabel();
    zoomLbl->setStyleSheet("QLabel{ margin: 5px; padding: 10px; background-color: rgba(0, 0, 0, 0.5); border-radius: 3px; }");
    zoomLbl->setText("Zoom: 1.0x");
    ui->gridLayout->addWidget(zoomLbl, 1, 2, 1, 1, Qt::AlignLeft | Qt::AlignTop);

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }

    // all toolbuttons icon size in toolBar
    for (QToolButton *b : ui->toolBar->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(toolBarIconSize);
        }
    }

	// set stylesheet
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());
    ui->sideView->setFixedWidth(sideViewSize);

    ui->appTitle->setAttribute(Qt::WA_TransparentForMouseEvents);
    ui->appTitle->setFocusPolicy(Qt::NoFocus);
    this->resize(800, 500);

    if (touch) {
        this->setWindowState(Qt::WindowMaximized);
    }

    ui->page->setCurrentIndex(0);

    connect(ui->viewWidget, &view::zoomChanged, this, &coreimage::changeZoomLabel);
    connect(ui->cOpen, SIGNAL(clicked()), this, SLOT(openFileDialog()));
    connect(ui->addFiles, SIGNAL(clicked()), this, SLOT(openFileDialog()));
    connect(ui->viewWidget, SIGNAL(nextImage()), this, SLOT(nextItem()));
    connect(ui->viewWidget, SIGNAL(prevImage()), this, SLOT(previousItem()));
    connect(ui->cNext, SIGNAL(clicked()), this, SLOT(nextItem()));
    connect(ui->cPrevious, SIGNAL(clicked()), this, SLOT(previousItem()));
    connect(ui->cZoomIn, SIGNAL(clicked()), ui->viewWidget, SLOT(zoomIn()));
    connect(ui->cZoomOut, SIGNAL(clicked()), ui->viewWidget, SLOT(zoomOut()));
    connect(ui->cNormalSize, SIGNAL(clicked()), ui->viewWidget, SLOT(zoomNormal()));
    connect(ui->menu, SIGNAL(clicked()), this, SLOT(showSideView()));
    connect(ui->appTitle, SIGNAL(clicked()), this, SLOT(showSideView()));
}

void coreimage::startMode()
{
    enableButtons(false);

    ui->cOpen->setEnabled(true);
    ui->menu->setEnabled(true);
    ui->sideView->setVisible(false);
}

void coreimage::shortcuts()
{
	QShortcut *shortcut;
    shortcut = new QShortcut(QKeySequence(Qt::Key_Left), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::previousItem);

    shortcut = new QShortcut(QKeySequence(Qt::Key_Right), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::nextItem);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_O), this);
    connect(shortcut, &QShortcut::activated, this, &coreimage::openFileDialog);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_S), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::on_cSave_clicked);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::SHIFT + Qt::Key_S), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::on_cSaveAs_clicked);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_D), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::on_cTrashIt_clicked);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_Plus), ui->app);
    connect(shortcut, &QShortcut::activated,  ui->viewWidget, &view::zoomIn);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_Minus), ui->app);
    connect(shortcut, &QShortcut::activated,  ui->viewWidget, &view::zoomOut);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_0), ui->app);
    connect(shortcut, &QShortcut::activated,  ui->viewWidget, &view::zoomNormal);

    shortcut = new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_B), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::on_pinIt_clicked);

    shortcut = new QShortcut(QKeySequence(Qt::Key_Escape), ui->app);
    connect(shortcut, &QShortcut::activated, this, &coreimage::closeSlideShow);
}

void coreimage::getImageFilters()
{
    openFilters.clear();
    saveFilters.clear();

    /* Open Filters */
    openFilters << "All Images (";

    Q_FOREACH (QByteArray type, QImageReader::supportedImageFormats()) {
        openFilters[ 0 ] += "*." + QString(type) + " ";
        openFilters << QString("%1 File (*.").arg(QString(type.toUpper())) + type + ")";
    }

    openFilters[ 0 ] += ")";

    /* Save Filters */
    saveFilters << "All Images (";

    Q_FOREACH (QByteArray type, QImageWriter::supportedImageFormats()) {
        saveFilters[ 0 ] += "*." + QString(type) + " ";
        saveFilters << QString("%1 File (*.").arg(QString(type.toUpper())) + type + ")";
    }

    saveFilters[ 0 ] += ")";
}

void coreimage::initializeImageFileDialog(QFileDialog &dialog, QFileDialog::AcceptMode acceptMode)
{
    const QStringList picturesLocations = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
    dialog.setDirectory(picturesLocations.isEmpty() ? QDir::currentPath() : picturesLocations.last());

    dialog.setAcceptMode(acceptMode);

    if (acceptMode == QFileDialog::AcceptSave) {
        dialog.setNameFilters(saveFilters);
        dialog.setDefaultSuffix(dialog.selectedNameFilter());
    } else {
        dialog.setNameFilters(openFilters);
    }
}

void coreimage::enableButtons(bool enable)
{
    for (QToolButton *b : ui->toolBar->findChildren<QToolButton *>()) {
        b->setEnabled(enable);
    }

    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        b->setEnabled(enable);
    }
}

void coreimage::getImagesForOpenedImage()
{
    if (!workFilePath.count()) {
        return;
    }

    QFileInfo fileInfo(workFilePath);
    QDir dir(fileInfo.path());

    if (!dir.exists()) {
        return;
    }

    if (imagesList.count()) {
        return;
    }

    QList<QByteArray> imageMime = QImageReader::supportedImageFormats();

    for (QFileInfo info : dir.entryInfoList(QDir::Files | QDir::NoDotAndDotDot | QDir::Readable)) {
        if (imageMime.contains(info.suffix().toLocal8Bit().toLower())) {
            imagesList.append(info.filePath());
            qApp->processEvents();
        }
    }
}

void coreimage::on_cSave_clicked()
{
    if (ui->viewWidget->saveImage()) {
        CPrime::InfoFunc::messageEngine("CoreImage", "coreimage", "Image Saved", "Check the destination for the file.", this);
	}
}

void coreimage::on_cSaveAs_clicked()
{
    QFileDialog dialog(this, tr("Save File As"));
    initializeImageFileDialog(dialog, QFileDialog::AcceptSave);

    while (dialog.exec() == QDialog::Accepted && !ui->viewWidget->saveImage(dialog.selectedFiles().first())) {}
}

void coreimage::on_rotate_clicked()
{
    ui->viewWidget->rotateRight();
}

void coreimage::on_openInEditor_clicked()
{
    CPrime::AppOpenFunc::defaultAppEngine(CPrime::Category::ImageEditor, workFilePath, "", this);
}

void coreimage::on_openmetadataApp_clicked()
{
    CPrime::AppOpenFunc::defaultAppEngine(CPrime::Category::MetadataViewer, workFilePath, "", this);
}

void coreimage::on_containingFolder_clicked()
{
    CPrime::AppOpenFunc::defaultAppEngine(CPrime::Category::FileManager,  CPrime::FileUtils::dirName( workFilePath ), "", this);
}

void coreimage::on_cTrashIt_clicked()
{
    if (!workFilePath.count()) {
        return;
    }

    int index = imagesList.indexOf(workFilePath);

    // Function from utilities.cpp
    if (CPrime::TrashManager::moveToTrash(QStringList() << workFilePath) == true) {
        imagesList.removeAt(index);

        if (!imagesList.count()) {
            ui->viewWidget->scene()->clear();
            ui->viewWidget->clear();
            ui->propertisesL->clear();
            startMode();

            this->setWindowTitle("CoreImage");
        } else if (imagesList.count() > 0) {
            if (imagesList.count() == 1) {
                workFilePath = imagesList.at(0);
                loadImage(workFilePath);
            } else if (index == imagesList.count() - 1) {
                workFilePath = imagesList.at(index - 1);
                previousItem();
            } else {
                workFilePath = imagesList.at(index);
                nextItem();
            }
        }
    }
}

void coreimage::on_shearIt_clicked()
{
    if (workFilePath.count()) {
        shareit::shareFiles(this, QStringList() << workFilePath);
    }
}

void coreimage::on_pinIt_clicked()
{
    if (workFilePath.count()) {
        pinit::callPinIt(this, workFilePath);
    }
}

void coreimage::on_slideShow_clicked(bool checked)
{
    if (checked) {
		ui->viewWidget->zoomFit();

        if (!slideShowTimer) {
			slideShowTimer = new QTimer();

			// switch to the next image when timeout
            connect(slideShowTimer, &QTimer::timeout, this, &coreimage::nextItem);
		}

        slideShowTimer->start(3000);
        ui->sideView->setVisible(false);

        if (ui->menu->isChecked()) {
            ui->menu->setChecked(false);
        }

        slideShow = true;
    } else {
        if (slideShowTimer) {
			delete slideShowTimer;
			slideShowTimer = nullptr;
            slideShow = false;
		}
	}

    this->setWindowState(this->windowState() ^ Qt::WindowFullScreen);
}

void coreimage::previousItem()
{
    if (imagesList.count() <= 1) {
        return;
    }


    // first do this to get all the space for contents
    if (touch) {
        if (not sm->value("CoreApps", "TabletMode")){
            ui->sideView->setVisible(0);
        }
    }

    int index = imagesList.indexOf(workFilePath);
    int newIndex = index + 1;

    /* Loop through all images, so that we can load a proper one */
    while (newIndex != index) {
        workFilePath = imagesList.value(newIndex);

        if (!workFilePath.count() or !loadImage(workFilePath)) {
            newIndex++;

            if (newIndex >= imagesList.count()) {
                newIndex = 0;
            }
        } else {
            index = newIndex;
//            break;
        }
    }
}

void coreimage::nextItem()
{
    if (imagesList.count() <= 1) {
        return;
    }

    // first do this to get all the space for contents
    if (touch) {
        if (not sm->value("CoreApps", "TabletMode")){
            ui->sideView->setVisible(0);
        }
    }

    int index = imagesList.indexOf(workFilePath);
    int newIndex = index - 1;

    /* Loop through all images, so that we can load a proper one */
    while (newIndex != index) {
        workFilePath = imagesList.value(newIndex);

        if (!workFilePath.count() or !loadImage(workFilePath)) {
            newIndex--;

            if (newIndex < 0) {
                newIndex = imagesList.count() - 1;
            }
        } else {
            index = newIndex;
//            break;
        }
    }
}

void coreimage::openFileDialog()
{
    QFileDialog dialog(this, tr("Open File"));
    initializeImageFileDialog(dialog, QFileDialog::AcceptOpen);

    if (dialog.exec() == QDialog::Accepted) {
        imagesList.clear();
        loadImage(dialog.selectedFiles().first());
        enableButtons(true);
        getImagesForOpenedImage();
    }
}

void coreimage::showSideView()
{
    if (ui->sideView->isVisible()) {
        ui->sideView->setVisible(false);
    } else {
        ui->sideView->setVisible(true);
    }
}

void coreimage::changeZoomLabel()
{
    zoomLbl->setText("Zoom: " + QString::number(ui->viewWidget->zoomFactor(), 'f', 2) + "x");
}

void coreimage::closeSlideShow()
{
    if (slideShow) {
        on_slideShow_clicked(false);
        ui->slideShow->setChecked(false);
    }
}
