/*
    *
    * This file is a part of CoreImage.
    * An image viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 2 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include "view.h"

view::view(QWidget *parent) : QGraphicsView(parent)
{
	setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
    setCacheMode(QGraphicsView::CacheBackground);
    setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
    setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
    setDragMode(QGraphicsView::ScrollHandDrag);

    image = new QGraphicsPixmapItem();
    image->setTransformationMode(Qt::SmoothTransformation);

	mScene = new QGraphicsScene();
    mScene->addItem(image);

    setScene(mScene);
}

bool view::loadImage(QString fileName)
{
	mFileName = fileName;

    QPixmap pix(fileName);

    if (pix.isNull()) {
		return false;
    }

    image->setPixmap(pix);
    mScene->setSceneRect(image->boundingRect());
	zoomFit();

    return true;
}

void view::clear()
{
    isFit = false;
}

QImage view::asImage()
{
	return image->pixmap().toImage();
}

qreal view::zoomFactor() {

	return mScale;
}

void view::zoomIn()
{
    if (mScale < 10.00) {
        isFit = false;

        scale(scaleFactor, scaleFactor);
        mScale *= scaleFactor;
        zoomChanged();
    }
}

void view::zoomOut()
{
    if (mScale > 0.05) {
        isFit = false;

        scale(invScaleFactor, invScaleFactor);
        mScale *= invScaleFactor;
        zoomChanged();
    }
}

void view::zoomNormal()
{
	resetTransform();
	isFit = false;
	mScale = 1.0;
	zoomChanged();
}

void view::zoomFit()
{
    fitInView(image, Qt::KeepAspectRatio);
	isFit = true;
	mScale = transform().m11();

	zoomChanged();
}

void view::rotateLeft()
{
	QTransform tran;
    tran.rotate(-90);

    mScene->removeItem(image);
    image->setPixmap(image->pixmap().transformed(tran, Qt::SmoothTransformation));
    mScene->addItem(image);
    mScene->setSceneRect(image->boundingRect());

    if (isFit) {
		zoomFit();
    }
}

void view::rotateRight()
{
	QTransform tran;
    tran.rotate(90);

    mScene->removeItem(image);
    image->setPixmap(image->pixmap().transformed(tran, Qt::SmoothTransformation));
    mScene->addItem(image);
    mScene->setSceneRect(image->boundingRect());

    if (isFit) {
		zoomFit();
    }
}

bool view::saveImage(QString fileName)
{
    if (fileName.count()) {
		mFileName = fileName;
    }

    return image->pixmap().save(mFileName);
}

void view::mouseDoubleClickEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        if (isFit) {
			zoomNormal();
        } else {
			zoomFit();
        }
    }
}

void view::wheelEvent(QWheelEvent *event)
{
	QPoint wheelScroll = event->angleDelta();

	if ((event->buttons() & Qt::RightButton) // Mouse right button + wheel zoom
			|| (event->modifiers() & Qt::ControlModifier)) { // Ctrl + wheel zoom
		if ((wheelScroll.x() > 0) || (wheelScroll.y() > 0)) {
			zoomIn();
		} else {
			zoomOut();
		}
	} else if (!event->buttons()) { // Navigation of images
		if ((wheelScroll.x() > 0) || (wheelScroll.y() > 0)) {
			emit nextImage();
		} else {
			emit prevImage();
		}
    } else {
        QGraphicsView::wheelEvent(event);
    }
}

void view::resizeEvent(QResizeEvent *rEvent)
{
    rEvent->accept();
    if (isFit) {
        zoomFit();
    }
}
